import { useEffect, useMemo, useRef, useState } from "react";
import { Graph } from "@antv/x6";
import { History } from "@antv/x6-plugin-history";
import { Transform } from "@antv/x6-plugin-transform";
import { Scroller } from "@antv/x6-plugin-scroller";
import "@/components/TableNode";
import { GetAllDesignTables } from "@/api";
import { useRequest } from "umi";
import type { ProjectInfo, TableItemType } from "@/type";
import { uuid } from "@/utils";
export default function erModel() {
  const graphRef = useRef<Graph>();
  const [graph, setGraph] = useState<Graph>();
  const [project, setProject] = useState<ProjectInfo>({
    id: "1",
    name: "项目1",
    folderId: "root",
    creationTime: "2024-12-12 12:12:12",
    creatorUser: "管理员",
    lastModificationTime: "2024-12-12 12:12:12",
    description: "模型描述",
    isTemplate: false,
    industry: "互联网",
    publishStatus: "未发布",
    tables: [],
    relations: [],
    TopicAreas: [],
    remarks: [],
    history: [],
    setting: {
      showMenu: true,
      showSidebar: true,
      showColumnDetail: true,
      showGrid: true,
      showRelation: true,
    },
  });

  const initGraph = (container: HTMLElement) => {
    const instance = new Graph({
      container,
      width: document.documentElement.clientWidth,
      height: document.documentElement.clientHeight,
      autoResize: true,
      async: false,
      mousewheel: {
        enabled: true,
        modifiers: "ctrl",
        minScale: 0.2,
        maxScale: 2,
      },
      connecting: {
        connectionPoint: "anchor",
      },
      grid: {
        visible: true,
        size: 10,
      },
      background: {
        color: "#F2F7FA",
      },
      interacting: {
        nodeMovable: (view) => {
          const data = view.cell.getData<{
            ignoreDrag: boolean;
            lock: boolean;
          }>();
          // 禁止拖拽或锁节点
          if (data?.ignoreDrag || data?.lock) return false;
          return true;
        },
      },
    });

    instance.use(new History());
    instance.use(
      new Transform({
        resizing: {
          enabled: true,
        },

      })
    );
    instance.use(new Scroller());

    setGraph(instance);
    graphRef.current = instance;
  };

  const { data, run, loading } = useRequest(
    () => GetAllDesignTables({ groupType: "" }),
    {
      manual: true,
    }
  );

  /**
   * 添加表
   */
  const addTable = () => {
    const tableId = uuid();
    const newTable: TableItemType = {
      table: {
        aliasName: "newtable",
        creationTime: "",
        creatorUserId: "",
        displayOrder: 1,
        id: tableId,
        isDeleted: false,
        langDescription: "",
        langName: "",
        name: "",
        cn_name: "新建表",
        en_name: "new table",
        parentBusinessTableId: "",
        schemaName: "new_table",
        type: 1,
        updateTime: "",
        openSync: false,
        style: {
          color: "#616161",
          x: 0,
          y: 0,
        },
      },
      tableColumnList: [
        {
          aggregateEnable: false,
          businessTableId: tableId,
          charset: "",
          defaultValue: "",
          displayEnable: false,
          displayOrder: 1,
          groupByEnable: false,
          id: uuid(),
          isAggregateEnable: false,
          isDisplayEnable: false,
          isGroupByEnable: false,
          isLinkEnable: false,
          isOrderByEnable: false,
          isPreDefined: true,
          isRequired: true,
          isUnique: false,
          isWhereEnable: false,
          langName: "120d5c4e-a5d8-4d49-b5f5-492feb37e6c6",
          linkEnable: false,
          maxLength: 50,
          name: "Id",
          orderByEnable: false,
          preDefined: true,
          precision: 0,
          required: true,
          scale: 0,
          schemaName: "id",
          type: 2,
          unique: false,
          whereEnable: false,
          cn_name: "id",
          en_name: "id",
        },
      ],
    };

    setProject({
      ...project,
      tables: [...project.tables, newTable],
    });

    graphRef.current?.addNode({
      shape: "table-node",
      x: 300,
      y: 100,
      width: 200,
      height: 200,
      id: tableId,
      data: newTable,
      zIndex: 1,
    });
  };

  /**
   * 更新表
   * @param table 
   */
  const updateTable = (table: TableItemType) => {
    setProject({
      ...project,
      tables: project.tables.map((item) => {
        if (item.table.id === table.table.id) {
          const tableNode = graphRef.current?.getCellById(table.table.id);
          tableNode?.setData(table);
          return table;
        }
        return item;
      }),
    });
  }

  /**
   * 删除表
   * @param tableId 
   */
  const deleteTable = (tableId: string) => {
    setProject({
      ...project,
      tables: project.tables.filter((item) => item.table.id !== tableId),
    });
    graphRef.current?.removeCell(tableId);
    // todo删除关联关系
  };

  return {
    initGraph,
    graph,
    graphRef,
    loading,
    project,
    setProject,
    addTable,
    updateTable,
    deleteTable,
  };
}
